@@ -41,6 +41,7 @@ type PipelineGenRet = (Element, AppSink, Receiver<ImageBuffer<Rgb<u8>, Vec<u8>>>
 /// # Quirks
 /// - `Drop`-ing this may cause a `panic`.
 /// - Setting controls is not supported.
+#[cfg_attr(feature = "docs-features", doc(cfg(feature = "input-gst")))]
 pub struct GStreamerCaptureDevice {
     pipeline: Element,
     app_sink: AppSink,
@@ -78,7 +79,7 @@ impl GStreamerCaptureDevice {
                 Err(why) => {
                     return Err(NokhwaError::GeneralError(format!(
                         "Failed to generate caps: {}",
-                        why.to_string()
+                        why
                     )))
                 }
             };
@@ -223,7 +224,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
                                                 property: "Framerates".to_string(),
                                                 error: format!(
                                                     "Failed to make framerates into string: {}",
-                                                    why.to_string()
+                                                    why
                                                 ),
                                             });
                                         }
@@ -260,7 +261,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
                                                     property: "Framerates".to_string(),
                                                     error: format!(
                                                         "Failed to parse numerator: {}",
-                                                        why.to_string()
+                                                        why
                                                     ),
                                                 });
                                             }
@@ -311,7 +312,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
                                                 property: "Framerates".to_string(),
                                                 error: format!(
                                                     "Failed to make framerates into string: {}",
-                                                    why.to_string()
+                                                    why
                                                 ),
                                             });
                                         }
@@ -348,7 +349,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
                                                     property: "Framerates".to_string(),
                                                     error: format!(
                                                         "Failed to parse numerator: {}",
-                                                        why.to_string()
+                                                        why
                                                     ),
                                                 });
                                             }
@@ -475,7 +476,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
         if let Err(why) = self.pipeline.set_state(State::Playing) {
             return Err(NokhwaError::OpenStreamError(format!(
                 "Failed to set appsink to playing: {}",
-                why.to_string()
+                why
             )));
         }
         Ok(())
@@ -532,7 +533,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
                 MessageView::Error(err) => {
                     return Err(NokhwaError::ReadFrameError(format!(
                         "Bus error: {}",
-                        err.error().to_string()
+                        err.error()
                     )));
                 }
                 _ => {}
@@ -544,7 +545,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
             Err(why) => {
                 return Err(NokhwaError::ReadFrameError(format!(
                     "Receiver Error: {}",
-                    why.to_string()
+                    why
                 )));
             }
         }
@@ -554,7 +555,7 @@ impl CaptureBackendTrait for GStreamerCaptureDevice {
         if let Err(why) = self.pipeline.set_state(State::Null) {
             return Err(NokhwaError::StreamShutdownError(format!(
                 "Could not change state: {}",
-                why.to_string()
+                why
             )));
         }
         Ok(())
@@ -616,7 +617,7 @@ fn generate_pipeline(fmt: CameraFormat, index: usize) -> Result<PipelineGenRet,
                     format!(
                         "Failed to open pipeline with args {}: {}",
                         webcam_pipeline(format!("{}", index).as_str(), fmt),
-                        why.to_string()
+                        why
                     ),
                 ))
             }
@@ -672,8 +673,7 @@ fn generate_pipeline(fmt: CameraFormat, index: usize) -> Result<PipelineGenRet,
                         element_error!(
                             appsink,
                             ResourceError::Failed,
-                            (format!("Failed to get videoinfo from caps: {}", why.to_string())
-                                .as_str())
+                            (format!("Failed to get videoinfo from caps: {}", why).as_str())
                         );
 
                         return Err(FlowError::Error);
@@ -697,8 +697,7 @@ fn generate_pipeline(fmt: CameraFormat, index: usize) -> Result<PipelineGenRet,
                         element_error!(
                             appsink,
                             ResourceError::Failed,
-                            (format!("Failed to map buffer to readablemap: {}", why.to_string())
-                                .as_str())
+                            (format!("Failed to map buffer to readablemap: {}", why).as_str())
                         );
 
                         return Err(FlowError::Error);
@@ -715,11 +714,7 @@ fn generate_pipeline(fmt: CameraFormat, index: usize) -> Result<PipelineGenRet,
                                 element_error!(
                                     appsink,
                                     ResourceError::Failed,
-                                    (format!(
-                                        "Failed to make yuy2 into rgb888: {}",
-                                        why.to_string()
-                                    )
-                                    .as_str())
+                                    (format!("Failed to make yuy2 into rgb888: {}", why).as_str())
                                 );
 
                                 return Err(FlowError::Error);
@@ -781,11 +776,7 @@ fn generate_pipeline(fmt: CameraFormat, index: usize) -> Result<PipelineGenRet,
                                 element_error!(
                                     appsink,
                                     ResourceError::Failed,
-                                    (format!(
-                                        "Failed to make yuy2 into rgb888: {}",
-                                        why.to_string()
-                                    )
-                                    .as_str())
+                                    (format!("Failed to make yuy2 into rgb888: {}", why).as_str())
                                 );
 
                                 return Err(FlowError::Error);
