@@ -123,7 +123,7 @@ pub fn try_known_camera_control_to_id(ctrl: KnownCameraControls) -> Option<u32>
         KnownCameraControls::Pan => Some(10_094_852),
         KnownCameraControls::Tilt => Some(100_948_530),
         KnownCameraControls::Zoom => Some(10_094_862),
-        KnownCameraControls::Exposure => Some(99_637_930),
+        KnownCameraControls::Exposure => Some(10_094_850),
         KnownCameraControls::Iris => Some(10_094_866),
         KnownCameraControls::Focus => Some(10_094_859),
         _ => None,
@@ -146,7 +146,7 @@ pub fn try_id_to_known_camera_control(id: u32) -> Option<KnownCameraControls> {
         10_094_852 => Some(KnownCameraControls::Pan),
         100_948_530 => Some(KnownCameraControls::Tilt),
         10_094_862 => Some(KnownCameraControls::Zoom),
-        99_637_930 => Some(KnownCameraControls::Exposure),
+        10_094_850 => Some(KnownCameraControls::Exposure),
         10_094_866 => Some(KnownCameraControls::Iris),
         10_094_859 => Some(KnownCameraControls::Focus),
         _ => None,
@@ -168,6 +168,7 @@ fn clone_control(ctrl: &Control) -> Control {
 /// - The `Any` return type for [`raw_supported_camera_controls()`](CaptureBackendTrait::raw_supported_camera_controls) is [`Description`]
 /// - The `Any` type for [`raw_camera_control()`](CaptureBackendTrait::raw_camera_control) is [`u32`], and its return `Any` is a [`Control`]
 /// - The `Any` type for `control` for [`set_raw_camera_control()`](CaptureBackendTrait::set_raw_camera_control) is [`u32`] and [`Control`]
+#[cfg_attr(feature = "docs-features", doc(cfg(feature = "input-v4l")))]
 pub struct V4LCaptureDevice<'a> {
     camera_format: CameraFormat,
     camera_info: CameraInfo,
@@ -187,7 +188,7 @@ impl<'a> V4LCaptureDevice<'a> {
             Err(why) => {
                 return Err(NokhwaError::OpenDeviceError(
                     index.to_string(),
-                    format!("V4L2 Error: {}", why.to_string()),
+                    format!("V4L2 Error: {}", why),
                 ))
             }
         };
@@ -378,7 +379,7 @@ impl<'a> CaptureBackendTrait for V4LCaptureDevice<'a> {
                     // undo
                     if let Err(why) = Capture::set_format(&self.device, &prev_format) {
                         return Err(NokhwaError::SetPropertyError {
-                            property: format!("Attempt undo due to stream acquisition failure with error {}. Resolution, FrameFormat", why.to_string()),
+                            property: format!("Attempt undo due to stream acquisition failure with error {}. Resolution, FrameFormat", why),
                             value: prev_format.to_string(),
                             error: why.to_string(),
                         });
@@ -386,7 +387,7 @@ impl<'a> CaptureBackendTrait for V4LCaptureDevice<'a> {
                     if let Err(why) = Capture::set_params(&self.device, &prev_fps) {
                         return Err(NokhwaError::SetPropertyError {
                             property:
-                            format!("Attempt undo due to stream acquisition failure with error {}. Frame rate", why.to_string()),
+                            format!("Attempt undo due to stream acquisition failure with error {}. Frame rate", why),
                             value: prev_fps.to_string(),
                             error: why.to_string(),
                         });
@@ -542,6 +543,10 @@ impl<'a> CaptureBackendTrait for V4LCaptureDevice<'a> {
                 };
 
                 for ctrl in v4l2_controls {
+                    if ctrl.id != id {
+                        continue;
+                    }
+
                     if let Ok(cam_control) = to_camera_control(&self.device, &ctrl) {
                         if Some(ctrl.id) == try_known_camera_control_to_id(cam_control.control()) {
                             return Ok(cam_control);
